@@ -1,7 +1,7 @@
 /** @file remote-database.cc
  *  @brief Remote backend database class
  */
-/* Copyright (C) 2006,2007,2008,2009,2010,2011,2012,2013,2014,2015,2019 Olly Betts
+/* Copyright (C) 2006,2007,2008,2009,2010,2011,2012,2013,2014,2015,2019,2020 Olly Betts
  * Copyright (C) 2007,2009,2010 Lemur Consulting Ltd
  *
  * This program is free software; you can redistribute it and/or
@@ -372,6 +372,8 @@ RemoteDatabase::update_stats(message_type msg_code, const string & body) const
 	    errmsg += "s ";
 	    errmsg += str(protocol_major);
 	    errmsg += ".0 to ";
+	} else {
+	    errmsg += ' ';
 	}
 	errmsg += str(protocol_major);
 	errmsg += '.';
@@ -633,10 +635,21 @@ RemoteDatabase::do_close()
     // it here.
     bool writable = (transaction_state != TRANSACTION_UNIMPLEMENTED);
 
-    // Only call dtor_called() if we're writable.
-    if (writable) dtor_called();
-
     if (writable) {
+	try {
+	    if (transaction_active()) {
+		cancel_transaction();
+	    } else {
+		commit();
+	    }
+	} catch (...) {
+	    try {
+		link.do_close();
+	    } catch (...) {
+	    }
+	    throw;
+	}
+
 	// If we're writable, send a shutdown message to the server and wait
 	// for it to close its end of the connection so we know that changes
 	// have been written and flushed, and the database write lock released.
@@ -755,22 +768,30 @@ RemoteDatabase::get_mset(Xapian::MSet &mset,
 void
 RemoteDatabase::commit()
 {
+    if (!uncommitted_changes) return;
+
     send_message(MSG_COMMIT, string());
 
     // We need to wait for a response to ensure documents have been committed.
     string message;
     get_message(message, REPLY_DONE);
+
+    uncommitted_changes = false;
 }
 
 void
 RemoteDatabase::cancel()
 {
+    if (!uncommitted_changes) return;
+
     cached_stats_valid = false;
     mru_slot = Xapian::BAD_VALUENO;
 
     send_message(MSG_CANCEL, string());
     string dummy;
     get_message(dummy, REPLY_DONE);
+
+    uncommitted_changes = false;
 }
 
 Xapian::docid
@@ -778,6 +799,7 @@ RemoteDatabase::add_document(const Xapian::Document & doc)
 {
     cached_stats_valid = false;
     mru_slot = Xapian::BAD_VALUENO;
+    uncommitted_changes = true;
 
     send_message(MSG_ADDDOCUMENT, serialise_document(doc));
 
@@ -796,6 +818,7 @@ RemoteDatabase::delete_document(Xapian::docid did)
 {
     cached_stats_valid = false;
     mru_slot = Xapian::BAD_VALUENO;
+    uncommitted_changes = true;
 
     send_message(MSG_DELETEDOCUMENT, encode_length(did));
     string dummy;
@@ -807,6 +830,7 @@ RemoteDatabase::delete_document(const std::string & unique_term)
 {
     cached_stats_valid = false;
     mru_slot = Xapian::BAD_VALUENO;
+    uncommitted_changes = true;
 
     send_message(MSG_DELETEDOCUMENTTERM, unique_term);
     string dummy;
@@ -819,6 +843,7 @@ RemoteDatabase::replace_document(Xapian::docid did,
 {
     cached_stats_valid = false;
     mru_slot = Xapian::BAD_VALUENO;
+    uncommitted_changes = true;
 
     string message = encode_length(did);
     message += serialise_document(doc);
@@ -834,6 +859,7 @@ RemoteDatabase::replace_document(const std::string & unique_term,
 {
     cached_stats_valid = false;
     mru_slot = Xapian::BAD_VALUENO;
+    uncommitted_changes = true;
 
     string message = encode_length(unique_term.size());
     message += unique_term;
@@ -868,6 +894,8 @@ RemoteDatabase::get_metadata(const string & key) const
 void
 RemoteDatabase::set_metadata(const string & key, const string & value)
 {
+    uncommitted_changes = true;
+
     string data = encode_length(key.size());
     data += key;
     data += value;
@@ -880,6 +908,8 @@ void
 RemoteDatabase::add_spelling(const string & word,
 			     Xapian::termcount freqinc) const
 {
+    uncommitted_changes = true;
+
     string data = encode_length(freqinc);
     data += word;
     send_message(MSG_ADDSPELLING, data);
@@ -891,6 +921,8 @@ void
 RemoteDatabase::remove_spelling(const string & word,
 				Xapian::termcount freqdec) const
 {
+    uncommitted_changes = true;
+
     string data = encode_length(freqdec);
     data += word;
     send_message(MSG_REMOVESPELLING, data);
