@@ -1,7 +1,7 @@
 /** @file closefrom.cc
  * @brief Implementation of closefrom() function.
  */
-/* Copyright (C) 2010,2011,2012,2016,2018 Olly Betts
+/* Copyright (C) 2010,2011,2012,2016,2018,2019 Olly Betts
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -37,11 +37,11 @@
 #if defined __linux__
 # include "alignment_cast.h"
 # include "safedirent.h"
-# include <cstdlib>
+# include "parseint.h"
 #elif defined __APPLE__
 # include <sys/attr.h>
-# include <cstdlib>
 # include <cstring>
+# include "parseint.h"
 #endif
 
 using namespace std;
@@ -115,11 +115,11 @@ Xapian::Internal::closefrom(int fd)
 	    for (ssize_t pos = 0; pos < c; pos += d->d_reclen) {
 		d = alignment_cast<struct dirent*>(buf + pos);
 		const char * leaf = d->d_name;
-		if (leaf[0] < '0' || leaf[0] > '9') {
+		int n;
+		if (!parse_signed(leaf, n)) {
 		    // Skip '.' and '..'.
 		    continue;
 		}
-		int n = atoi(leaf);
 		if (n < fd) {
 		    // FD below threshold.
 		    continue;
@@ -188,11 +188,11 @@ Xapian::Internal::closefrom(int fd)
 		const char * leaf = p + sizeof(u_int32_t);
 		p += *static_cast<u_int32_t*>(static_cast<void*>(p));
 
-		if (leaf[0] < '0' || leaf[0] > '9') {
+		int n;
+		if (!parse_signed(leaf, n)) {
 		    // Skip '.' and '..'.
 		    continue;
 		}
-		int n = atoi(leaf);
 		if (n < fd) {
 		    // FD below threshold.
 		    continue;
