@@ -384,33 +384,44 @@ DEFINE_TESTCASE(testlock1, chert || glass) {
     return true;
 }
 
-/** Test that locked() returns false for databases which don't support update.
+/** Test that locked() returns false for backends which don't support update.
  *
  *  Regression test for bug fixed in 1.4.6.
- *
- *  The remote backend doesn't support locked() currently.
- *
- *  An inmemory Database is always actually a WritableDatabase viewed as a
- *  Database, so it should always report being locked for writing - this is
- *  tested by testlock3 below.
  */
-DEFINE_TESTCASE(testlock2, backend && !remote && !inmemory) {
+DEFINE_TESTCASE(testlock2, backend && !writable) {
     Xapian::Database db = get_database("apitest_simpledata");
     TEST(!db.locked());
+    db.close();
+    TEST(!db.locked());
     return true;
 }
 
-/** Test that locked() returns true for inmemory Database objects.
+/** Test locked() on inmemory Database objects.
  *
  *  An inmemory Database is always actually a WritableDatabase viewed as a
- *  Database, so it should always report being locked for writing.
+ *  Database, so it should always report being locked for writing, unless
+ *  close() has been called.
  *
  *  Regression test for bug fixed in 1.4.14 - earlier versions always returned
  *  false for an inmemory Database here.
+ *
+ *  Regression test for bug fixed in 1.4.15 - false should be returned after
+ *  close() has been called.
  */
 DEFINE_TESTCASE(testlock3, inmemory) {
     Xapian::Database db = get_database("apitest_simpledata");
     TEST(db.locked());
+    db.close();
+    TEST(!db.locked());
+    return true;
+}
+
+/// Test locked() on closed WritableDatabase.
+DEFINE_TESTCASE(testlock4, glass) {
+    Xapian::Database db = get_writable_database("apitest_simpledata");
+    TEST(db.locked());
+    db.close();
+    TEST(!db.locked());
     return true;
 }
 
@@ -983,13 +994,14 @@ DEFINE_TESTCASE(emptydb1, backend) {
 	Xapian::Query::OP_FILTER,
 	Xapian::Query::OP_NEAR,
 	Xapian::Query::OP_PHRASE,
-	Xapian::Query::OP_ELITE_SET
+	Xapian::Query::OP_ELITE_SET,
+	Xapian::Query::OP_SYNONYM,
+	Xapian::Query::OP_MAX
     };
-    const Xapian::Query::op * p;
-    for (p = ops; p - ops != sizeof(ops) / sizeof(*ops); ++p) {
-	tout << *p << endl;
+    for (Xapian::Query::op op : ops) {
+	tout << op << endl;
 	Xapian::Enquire enquire(db);
-	Xapian::Query query(*p, Xapian::Query("a"), Xapian::Query("b"));
+	Xapian::Query query(op, Xapian::Query("a"), Xapian::Query("b"));
 	enquire.set_query(query);
 	Xapian::MSet mset = enquire.get_mset(0, 10);
 	TEST_EQUAL(mset.get_matches_estimated(), 0);
@@ -999,6 +1011,43 @@ DEFINE_TESTCASE(emptydb1, backend) {
     return true;
 }
 
+/** Test operators which should allow more than two arguments.
+ *
+ *  Regression test for bug with OP_FILTER fixed in 1.4.15, and also for bugs
+ *  with deleting the PostList which is currently set as the QueryOptimiser's
+ *  hint fixed in 1.4.15.
+ */
+DEFINE_TESTCASE(multiargop1, backend) {
+    Xapian::Database db(get_database("apitest_simpledata"));
+    static const struct { unsigned hits; Xapian::Query::op op; } tests[] = {
+	{ 0, Xapian::Query::OP_AND },
+	{ 6, Xapian::Query::OP_OR },
+	{ 0, Xapian::Query::OP_AND_NOT },
+	{ 5, Xapian::Query::OP_XOR },
+	{ 2, Xapian::Query::OP_AND_MAYBE },
+	{ 0, Xapian::Query::OP_FILTER },
+	{ 0, Xapian::Query::OP_NEAR },
+	{ 0, Xapian::Query::OP_PHRASE },
+	{ 6, Xapian::Query::OP_ELITE_SET },
+	{ 6, Xapian::Query::OP_SYNONYM },
+	{ 6, Xapian::Query::OP_MAX }
+    };
+    static const char* terms[] = {"two", "all", "paragraph", "banana"};
+    Xapian::Enquire enquire(db);
+    for (auto& test : tests) {
+	Xapian::Query::op op = test.op;
+	Xapian::doccount hits = test.hits;
+	tout << op << " should give " << hits << " hits\n";
+	Xapian::Query query(op, terms, terms + 4);
+	enquire.set_query(query);
+	Xapian::MSet mset = enquire.get_mset(0, 10);
+	TEST_EQUAL(mset.get_matches_estimated(), hits);
+	TEST_EQUAL(mset.get_matches_upper_bound(), hits);
+	TEST_EQUAL(mset.get_matches_lower_bound(), hits);
+    }
+    return true;
+}
+
 /// Test error opening non-existent stub databases.
 // Regression test for bug fixed in 1.3.1 and 1.2.11.
 DEFINE_TESTCASE(stubdb7, !backend) {
