@@ -134,7 +134,9 @@ get_loadavg()
     string loadavg;
     try {
 	loadavg = stdout_to_string("uptime 2>/dev/null | sed 's/.*: \\([0-9][0-9]*\\)/\\1/;s/, .*//'");
-    } catch (NoSuchProgram) {} catch (ReadError) {}
+    } catch (const NoSuchProgram&) {
+    } catch (const ReadError&) {
+    }
     return loadavg;
 #endif
 }
@@ -152,24 +154,32 @@ get_ncpus()
     try {
 	// Works on Linux, at least back to kernel 2.2.26.
 	ncpus = stdout_to_string("getconf _NPROCESSORS_ONLN 2>/dev/null | grep -v '[^0-9]'");
-    } catch (NoSuchProgram) {} catch (ReadError) {}
+    } catch (const NoSuchProgram&) {
+    } catch (const ReadError&) {
+    }
     if (ncpus.empty())
 	try {
 	    // Works on OpenBSD (and apparently FreeBSD and Darwin).
 	    ncpus = stdout_to_string("sysctl hw.ncpu 2>/dev/null | sed 's/.*=//'");
-	} catch (NoSuchProgram) {} catch (ReadError) {}
+	} catch (const NoSuchProgram&) {
+	} catch (const ReadError&) {
+	}
     if (ncpus.empty())
 	try {
 	    // Works on Solaris and OSF/1.
 	    ncpus = stdout_to_string("PATH=/usr/sbin:$PATH psrinfo 2>/dev/null | grep -c on-line");
-	} catch (NoSuchProgram) {} catch (ReadError) {}
+	} catch (const NoSuchProgram&) {
+	} catch (const ReadError&) {
+	}
     if (ncpus.empty())
 	try {
 	    // Works on Linux, just in case the getconf version doesn't.
 	    // Different architectures have different formats for /proc/cpuinfo
 	    // so this won't work as widely as getconf _NPROCESSORS_ONLN will.
 	    ncpus = stdout_to_string("grep -c processor /proc/cpuinfo 2>/dev/null");
-	} catch (NoSuchProgram) {} catch (ReadError) {}
+	} catch (const NoSuchProgram&) {
+	} catch (const ReadError&) {
+	}
 #endif
     return ncpus;
 }
@@ -193,7 +203,9 @@ get_distro()
 #else
     try {
 	distro = stdout_to_string("perftest/get_machine_info 2>/dev/null");
-    } catch (NoSuchProgram) {} catch (ReadError) {}
+    } catch (const NoSuchProgram&) {
+    } catch (const ReadError&) {
+    }
 #endif
     return distro;
 }
@@ -205,7 +217,9 @@ get_commit_ref()
     string commit_ref;
     try {
 	commit_ref = stdout_to_string("cd \"$srcdir\" && git log -n1 --abbrev-commit --format=%h");
-    } catch (NoSuchProgram) {} catch (ReadError) {}
+    } catch (const NoSuchProgram&) {
+    } catch (const ReadError&) {
+    }
 
     return commit_ref;
 }
